#include "tac_plus.h"
#include <ttyent.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <stdlib.h>
#include <strings.h>

#define MAXNAS 16

extern void login(UTMP_T *ut);

void reg_nas_users(void) {
    struct ttyent *ttyp;
    char *pnas[MAXNAS];
    int nnas=0;
    int j;

    if(!setttyent()) return;
    for(;(ttyp = getttyent());) {
	int i=0, j=0;
/* printf("name=%s, getty=%s, type=%s, status=%i, window=%s, comment=%s\n",ttyp->ty_name,ttyp->ty_getty,ttyp->ty_type,ttyp->ty_status,ttyp->ty_window,ttyp->ty_comment); */
	while(i<nnas)
	    if(!strcmp(pnas[i++],ttyp->ty_getty))
		j=1;
	if(!j) {
	    if (nnas++ < MAXNAS)
		pnas[nnas-1]=strdup(ttyp->ty_getty);
	    else
		report(LOG_WARNING, "reg_nas_users: max number of NAS=%i exceeded",MAXNAS);
	}
    }

for(j=0;j<nnas;j++) {
    char *nas=pnas[j]; /* pnas[j] contains found NAS */
    struct sockaddr_in sin;
    struct servent *serv;
    int s, bufsize;
    char *buf, *p, *pn;
    int incr = 4096, slop = 32;
    u_long inaddr;

    /*
     * The finger service, aka port 79
     */
    serv = getservbyname("finger", "tcp");
    if (serv) {
	sin.sin_port = serv->s_port;
    } else {
	sin.sin_port = 79;
    }

    /*
     * Get IP addr for the NAS
     */
    inaddr = inet_addr(nas);
    if (inaddr != -1) {
	/* A dotted decimal address */
	bcopy(&inaddr, &sin.sin_addr, sizeof(inaddr));
	sin.sin_family = AF_INET;
    } else {
	struct hostent *host = gethostbyname(nas);
	if (host == NULL) {
	    report(LOG_ERR, "reg_nas_users: gethostbyname %s failure: %s",
		   nas, strerror(errno));
	    continue;
	}
	bcopy(host->h_addr, &sin.sin_addr, host->h_length);
	sin.sin_family = host->h_addrtype;
    }

    s = socket(AF_INET, SOCK_STREAM, 0);
    if (s < 0) {
	report(LOG_ERR, "reg_nas_users: NAS %s socket: %s", nas, strerror(errno));
	continue;
    }

    if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
	report(LOG_ERR, "reg_nas_users: NAS %s connect failure %s", nas, strerror(errno));
	close(s);
	continue;
    }

    /*
     * Read in the finger output into a single flat buffer
     */
    buf = NULL;
    bufsize = 0;
    for (;;) {
	int x;

	buf = tac_realloc(buf, bufsize + incr + slop);
	x = read(s, buf + bufsize, incr);
	if (x <= 0) {
	    break;
	}
	bufsize += x;
    }

    /*
     * Done talking here
     */
    close(s);
    buf[bufsize] = '\0';


    /* skip first line in buffer */
    p = strchr(buf, '\n');
    if (p) {
	p++;
    }
    p = strchr(p, '\n');
    if (p) {
	p++;
    }

    /*
     * Tally each time this user appears
     */
    for (; p && *p; p = pn) {
	int i, len, nmlen;
	char nmbuf[11];
	char *p1,*p2;
	char ttyname[16]="tty";

	/*
	 * Find next line
	 */
	pn = strchr(p, '\n');
	if (pn) {
	    ++pn;
	}

	/*
	 * Calculate line length
	 */
	if (pn) {
	    len = pn - p;
	} else {
	    len = strlen(p);
	}

	/*
	 * Line too short -> ignore
	 */
	if (len < 14) {
	    continue;
	}

#if 0
	/*
	 * Always ignore the port were currently trying to login on.
	 */
	if (curport && isdigit(*curport)) {
	    int thisport;

	    if (sscanf(p + 1, " %d", &thisport) == 1) {
		if (atoi(curport) == thisport) {
		    continue;
		}
	    }
	}
#endif

	p1=p;
	while ((*p1 == ' ') || (*p1 == '*')) p1++;
	p2=strchr(p1,' ');
	strncat(ttyname,p1,p2-p1);

	/*
	 * Extract username, up to 10 chars wide, starting at char 13
	 */
	nmlen = 0;
	p += 13;
	for(i=0; *p && !isspace((int)*p) && (i < 10); i++) {
	    nmbuf[nmlen++] = *p++;
	}
	nmbuf[nmlen++] = '\0';

	if (nmbuf[0]) {
	    UTMP_T u;
	    bzero(&u, sizeof(u));
	    strncpy(u.ut_line, ttyname, UT_LINESIZE);
	    strncpy(u.ut_host, nas, UT_HOSTSIZE);
	    strncpy(u.ut_user, nmbuf, UT_NAMESIZE);
	    time(&u.ut_time);
	    report(LOG_INFO, "reg_nas_users: found user `%s' on NAS `%s' on `%s'",nmbuf,nas,ttyname);
	    login(&u);
	}
    }
    free(buf);
}

    for(j=0;j<nnas;j++)
	free(pnas[j]);

    endttyent();
}
