#include "tac_plus.h"
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#if defined(ALIASES) || defined(TTYS)
 #include "own_wtmp.h"
#endif

#ifdef DBF
#ifdef HAVE_DB_185_H
#include <db_185.h>
#else
#include <db.h>
#endif

#ifdef DBF_1
typedef struct {
    long tl;			/* time limit      */
    long tll;			/* time limit left */
    long dl;			/* day  limit      */
    long dll;			/* day  limit left */
    long tot;			/* total time      */
#ifdef PARROT
    double parrot;		/* parrots left    */
    double parrot_credit;
#endif
} TQ_st;
#endif

#endif

#define fill_pw()		\
  if(!pw)			\
  {				\
   pwdummy.pw_name = name;	\
   pwdummy.pw_uid = NOPUID;	\
   pwdummy.pw_gid = NOPGID;	\
   pwdummy.pw_dir = NOPHOME;	\
   pwdummy.pw_shell = NOPSHELL;	\
   pw = &pwdummy;		\
  }


int sys_adm_verify(name, pw, data, recurse)
char *name;
struct passwd *pw;
struct authen_data *data;
int recurse;
{
 struct passwd pwdummy;

 if (data->service == TAC_PLUS_AUTHEN_SVC_LOGIN)
  {
   char buff[4096];

   fill_pw();

#ifdef CRON
   if (set_time_status(name, data, pw, recurse))
     return 0;
#endif

#ifdef DENY
   if (set_sysadm_status(name, data, recurse))
     return 0;
#endif

#ifdef DBF
   if (set_usradm_status(name, data))
     return 0;
#endif

   snprintf(buff, sizeof(buff), "%s:%d:%d:%s:%s",
   pw->pw_name, pw->pw_uid, pw->pw_gid, pw->pw_dir, pw->pw_shell);
   data->server_data = tac_strdup(buff);
   data->server_dlen = strlen(buff)+1;
  }

#if (defined(CRON)  && (defined(CRON_FOR_ENABLE) || defined(CRON_FOR_PPP))) || \
    (defined(DENY)  && (defined(DENY_FOR_ENABLE) || defined(DENY_FOR_PPP))) || \
    (defined(DBF)   && (defined(DBF_FOR_ENABLE)  || defined(DBF_FOR_PPP)))

 else if(data->service == TAC_PLUS_AUTHEN_SVC_ENABLE
#ifdef PERUSER_ENABLE
     || data->service == TAC_PLUS_AUTHEN_SVC_ENUSER
#endif
#if defined(CRON_FOR_PPP) || defined(DENY_FOR_PPP) || defined(DBF_FOR_PPP)
     || data->service == TAC_PLUS_AUTHEN_SVC_PPP
#endif
        )
  {

#if defined(CRON) && (defined(CRON_FOR_ENABLE) || defined(CRON_FOR_PPP))
   fill_pw();
   if (set_time_status(name, data, pw, recurse))
     return 0;
#endif

#if defined(DENY) && (defined(DENY_FOR_ENABLE) || defined(DENY_FOR_PPP))
   if (set_sysadm_status(name, data, recurse))
     return 0;
#endif

#if defined(DBF) && (defined(DBF_FOR_ENABLE) || defined(DBF_FOR_PPP))
   if (set_usradm_status(name, data))
     return 0;
#endif

  }
#endif /* ... || ... */

  return (data->status == TAC_PLUS_AUTHEN_STATUS_PASS);
}


#ifdef DBF

int set_usradm_status(u, data)
char *u;
struct authen_data *data;
{
  static DB      *db;
  static DBT     key[1], val[1];

#ifdef DBF_1
  static TQ_st   *tqp;
  static char    unl[] = "unlimited";
  static char    buf1[16];
  static char    buf2[16];
#endif

  int retval = 0;
  char *s = tac_malloc(MAX_MSG_LEN);
  bzero(s, MAX_MSG_LEN);

  if (debug & DEBUG_AUTHEN_FLAG)
	report(LOG_DEBUG, "set_usradm_status: name=%s", u);

  db = dbopen(TAC_PLUS_DBFILE, O_RDONLY|O_EXLOCK, 0600, DB_HASH, NULL);
  if (!db) {
   if (debug & DEBUG_AUTHEN_FLAG)
     report(LOG_DEBUG, "set_usradm_status: no database file");
   return 0;
  }

  key->data = u;
  key->size = strlen(u)+1;
  if ((*db->get) (db, key, val, NULL)) {
    report(LOG_INFO, "set_usradm_status: can't get key value for %s", u);
#ifdef DBF_STRICT
    goto e1;
#endif
  }

  if (!val->data)
   {
#ifdef DBF_SILENT
    goto e2;
#else
    snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: time is unlimited.", u);
#endif
   }
  else
   {

#ifdef DBF_1

    tqp = (TQ_st *) val->data;

#ifdef PARROT
    if (tqp->parrot != PARROT_OFF) {
	if (tqp->parrot + tqp->parrot_credit <= 0.0) {
	   snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: parrot limit EXCEEDED!", u);
	   goto e1;
	}
	else
	   snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: left %7.2f parrots.", u, tqp->parrot);
      }
#endif
    if (tqp->dll <= 0) {
      if ((tqp->dl != TIME_LIMIT_NO) && (tqp->dl != TIME_LIMIT_OFF)) {
        snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: daily time limit EXCEEDED!", u);
	goto e1;
      }
    }
    if (tqp->tll <= 0) {
      if ((tqp->tl != TIME_LIMIT_NO) && (tqp->tl != TIME_LIMIT_OFF)) {
        snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: total time limit EXCEEDED!", u);
	goto e1;
      }
    }
    if ((tqp->dl == TIME_LIMIT_NO) || (tqp->dl == TIME_LIMIT_OFF))
      strncpy(buf1, unl, sizeof(buf1));
    else
      snprintf(buf1, sizeof(buf1), "%ld", tqp->dll / 60);
    if ((tqp->tl == TIME_LIMIT_NO) || (tqp->tl == TIME_LIMIT_OFF))
      strncpy(buf2, unl, sizeof(buf1));
    else
      snprintf(buf2, sizeof(buf2), "%ld", tqp->tll / 60);
    snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: total time left %s minutes, left for today %s minutes.",
	     u, buf2, buf1);

#else

    if (!*(time_t *)val->data)
     {
      snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: time limit EXCEEDED!", u);
      goto e1;
     }

    snprintf(s+strlen(s), MAX_MSG_LEN-strlen(s), "\n*** User %s: time left %d minutes.",
     	     u, *(time_t *)val->data / 60);
#endif
   }

  goto e0;

e1:
  data->status = TAC_PLUS_AUTHEN_STATUS_FAIL;
  retval = 1;

e0:
  fr_fi(data->server_msg, s);

#ifdef DBF_SILENT
e2:
#endif
  (*db->close) (db);
  if((debug & DEBUG_AUTHEN_FLAG) && retval)
    report(LOG_INFO, "user %s login not allowed by db rules", u);
  return retval;
}

#endif


#ifdef ALIASES

static char *nextvalid(p)
char **p;
{
  char	*ppp;

  while(**p && ((**p == ' ') || (**p == '\t'))) (*p)++;
  if(!*(ppp = *p) || *ppp == '#')
    return NULL;

  while(**p && (**p != ' ') && (**p != '\t')) (*p)++;
  if(**p)
   {
    **p = '\0';
    (*p)++;
   }

  return ppp;
}

static char *
value(p)
	char *p;
{

	return ((p = strchr(p, '=')) ? ++p : NULL);
}

char *get_peer_alias(peer, flags)
char *peer;
int *flags;
{
FILE	*f;
char  tmp[1024],
      *p;
char  *pp,
      *al,
      *s;
int tflag;
size_t u;

if ((f = fopen(TAC_PLUS_ALIAS_FILE, "r")) != NULL)
 {
  while(fgets(tmp, sizeof(tmp), f))
    {
      u = strlen(tmp);
      if(u && tmp[u - 1] == '\n')
        tmp[u - 1] = '\0';

      p = tmp;
      if((al = nextvalid(&p)) == NULL)	/* alias	*/
        continue;

      tflag = B_FLAG_INIT;			/* init flag	*/

#define	vcmp(e)	(!strncmp(pp, e, sizeof(e) - 1) && pp[sizeof(e) - 1] == '=')
#define mcmp(e) (!strncmp(s, e, sizeof(e) - 1))

					/* check flags		*/
      while((pp = nextvalid(&p)) != NULL) {

                if(!(s = value(pp)))	/* no value - no keyword */
                  break;

		     if(vcmp(_TTYS_CHECK))
		 {
		       if(mcmp(_TTYS_YES))
		         tflag |= A_FLAG_CHECK;
		  else if(mcmp(_TTYS_NO))
		         tflag &= ~A_FLAG_CHECK;
		 }
		else if(vcmp(_TTYS_PCHECK))
		 {
		       if(mcmp(_TTYS_YES))
		         tflag |= A_FLAG_PCHECK;
		  else if(mcmp(_TTYS_NO))
		         tflag &= ~A_FLAG_PCHECK;
		 }
		else if(vcmp(_TTYS_TTYS))
		 {
		       if(mcmp(_TTYS_YES))
		         tflag |= A_FLAG_TTYS;
		  else if(mcmp(_TTYS_NO))
		         tflag &= ~A_FLAG_TTYS;
		 }
		else if(vcmp(_TTYS_PUTMP) || vcmp(_TTYS_PWTMP))
		 {
		       if(mcmp(_TTYS_ALIAS))
		         tflag |= A_FLAG_PUTMP_AL;
		  else if(mcmp(_TTYS_TRUE))
		         tflag &= ~A_FLAG_PUTMP_AL;
		 }
		else if(vcmp(_TTYS_PACCT))
		 {
		       if(mcmp(_TTYS_ALIAS))
		         tflag |= A_FLAG_PACCT_AL;
		  else if(mcmp(_TTYS_TRUE))
		         tflag &= ~A_FLAG_PACCT_AL;
		 }
		else if(vcmp(_TTYS_UTMP) || vcmp(_TTYS_WTMP))
		 {
		       if(mcmp(_TTYS_ALIAS))
		        {
		         tflag &= ~A_FLAG_UTMP_X;
		         tflag |= A_FLAG_UTMP_AL;
		        }
		  else if(mcmp(_TTYS_TRUE))
		        {
		         tflag &= ~A_FLAG_UTMP_X;
		         tflag |= A_FLAG_UTMP_TR;
		        }
		  else if(mcmp(_TTYS_ASIS))
		        {
		         tflag &= ~A_FLAG_UTMP_X;
		         tflag |= A_FLAG_UTMP_AS;
		        }
		  else if(mcmp(_TTYS_NO))
		         tflag &= ~A_FLAG_UTMP_X;
		 }
		else if(vcmp(_TTYS_ACCT))
		 {
		       if(mcmp(_TTYS_ALIAS))
		        {
		         tflag &= ~A_FLAG_ACCT_X;
		         tflag |= A_FLAG_ACCT_AL;
		        }
		  else if(mcmp(_TTYS_TRUE))
		        {
		         tflag &= ~A_FLAG_ACCT_X;
		         tflag |= A_FLAG_ACCT_TR;
		        }
		  else if(mcmp(_TTYS_ASIS))
		        {
		         tflag &= ~A_FLAG_ACCT_X;
		         tflag |= A_FLAG_ACCT_AS;
		        }
		  else if(mcmp(_TTYS_NO))
		         tflag &= ~A_FLAG_ACCT_X;
		 }
		else
			break;
      }

						/* find peer	*/
      for(; pp; pp = nextvalid(&p))
       {
        if(!strcmp(peer, pp))
         {
          fclose(f);
          *flags = tflag;
          return tac_strdup(al);
         }
       }
    }
  fclose(f);
 }

*flags = B_FLAG_INIT;
return tac_strdup(peer);
}

#endif

#ifdef TTYS

char *get_port_alias(port, host, alias, flags, slot)
char *port, *host, *alias;
int *flags;
int *slot;
{
 char x[UT_LINESIZE + 1];
 char *trueline = x;

 *slot = t_ttyslot(port, host, alias, &trueline, flags);

 x[sizeof(x) - 1] = '\0';

 return tac_strdup(x);
}

#endif
